;;;; lookup-table-test.scm

(use test lookup-table)

;;;

;;

(define (dict-alist-test al tst)
  (let ((tbl1 (alist->dict al tst)))

    (test-assert (dict? tbl1))
    (test tst (dict-equivalence-function tbl1))

    (test-assert (tst 'bar (dict-ref tbl1 'foo)))
    (dict-delete! tbl1 'foo)
    (test-assert (not (dict-ref tbl1 'foo)))

    (test '() (dict-update! tbl1 'foo (lambda () '())))
    (test '(1) (dict-update! tbl1 'foo void (lambda (x) (append x '(1)))))

    (test '() (dict-update-list! tbl1 'list))
    (test '(1 2) (dict-update-list! tbl1 'list 1 2))

    (let ((tbl2 (dict-update-dict! tbl1 'dict)))
      (test-assert (dict? (dict-ref tbl1 'dict)))
      (test tbl2 (dict-ref tbl1 'dict))
      (test tbl2 (dict-update-dict! tbl1 'dict)) ) ) )

;;

;Checks new order for 'make-dict'
(define (dict-ht-test)
  (let ((tbl1 (make-dict equal? 2)))

    (do ([i 0 (add1 i)])
        ([> i 54])
      (if (odd? i)
          (dict-set! tbl1 i (->string i))
          (dict-set! tbl1 (->string i) i)))

    (test 20 (dict-ref tbl1 "20"))
    (dict-delete! tbl1 "20")
    (test-assert (not (dict-ref tbl1 "20")))

    (dict-merge! tbl1 (alist->dict '((foo . bar) (baz . bop)) equal?))

    (test 'bop (dict-ref tbl1 'baz))

    (test 'bar (dict-search tbl1 (lambda (key val) (eq? key 'foo))))

    (with-output-to-string (lambda () (dict-print tbl1))) ) )

;;;

(dict-safe-mode #t)
(newline) (print "** Alist Test (Safe eq?) **") (newline)
(dict-alist-test '((foo . bar) (baz . bop)) eq?)
(newline) (print "** Alist Test (Safe equal?) **") (newline)
(dict-alist-test '((foo . bar) (baz . bop) (2 . 3)) equal?)
(newline) (print "** HT Test (Safe) **") (newline)
(dict-ht-test)

(dict-safe-mode #f)
(newline) (print "** Alist Test (Unsafe eq?) **") (newline)
(dict-alist-test '((foo . bar) (baz . bop)) eq?)
(newline) (print "** Alist Test (Unsafe equal?) **") (newline)
(dict-alist-test '((foo . bar) (baz . bop) (2 . 3)) equal?)
(newline) (print "** HT Test (Unsafe) **") (newline)
(dict-ht-test)
